/*
 * Copyright (C) 2010 Canonical Ltd.
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Ken VanDine <ken.vandine@canonical.com>
 */


// BEGIN SAMPLE DATA
uint MAX_ROWS = 100;

Dee.Model
create_model ()
{
  var names = new string[10] { "Neil Jagdish Patel",
                               "Ken Van Dine",
                               "Ryan Paul",
                               "Paul Hummer",
                               "Gaurav Patel",
                               "Evan",
                               "Alexander Sack",
                               "Jorge Castro",
                               "Gustavo Niemeyer",
                               "Ted Gould"};

  var messages = new string[10] {
      "Adding unit tests to the Unity panel service. Mixing GTester + C++ is interesting, definitely not as bad as I thought it would be.",
      "The police officer that was shot in this news story <a href='http://is.gd/hmV87'>http://is.gd/hmV87</a> is a dear friend of mine, everyone send good vibes his way",
      "RT <a href='@webmink'>@webmink</a>: RT <a href='@BoingBoing'>@BoingBoing</a>: Odds of cancer from TSA scanners about the same as terrorist blowing up your plane <a href='http://bit.ly/9JKiPb'>http://bit.ly/9JKiPb</a>",
      "It's been so long since I've seen my wife that I enjoyed the TSA pat down a bit too much...",
      "Happy belated birthday <a href='@Whatleydude'>@Whatleydude</a>!",
      "There are far too many Roman columns in this part of town.",
      "back to basics: sunday house cleaning - bah!",
      "Goddamnit Smallville, it's the last season. Be relevant and not so ... sandbaggy. This \"mutant registration act\" thing is overdone.",
      "♺ <a href='@rbp'>@rbp</a>: RT <a href='@ThePSF'>@ThePSF</a> rt <a href='@voidspace'>@voidspace</a>: Here is the list of Python bugs closed (so far) during the bug weekend: Way to go!> <a href='http://goo.gl/V8cJu'>http://goo.gl/V8cJu</a>",
      "<a href='@bratschegnome'>@bratschegnome</a>: Cool! Fortunately Google knows more about bikes than I do :)"};

  var colors = new string[10] {
      "#4e4e9a9a0606",
      "#72729f9fcfcf",
      "#45100ebd5701",
      "#72729f9fcfcf",
      "#4e4e9a9a0606",
      "#45100ebd5701",
      "#72729f9fcfcf",
      "#45100ebd5701",
      "#4e4e9a9a0606",
      "#4e4e9a9a0606"
  };

  Dee.Model model;

  model = new Dee.SequenceModel ();
  model.set_schema ("u", "s", "s", "u", "s", "s", "s");

  for (uint i = 0; i < MAX_ROWS; i++)
    {
      model.append (i,
                    "Twitter",
                    names[i % 10],
                    0,
                    messages[i % 10],
                    "stock_person",
                    colors[i % 10]);
    }

  return model.ref () as Dee.Model;
}


// END SAMPLE DATA

public class Client : Gtk.Window
{
  public Client ()
  {
  }

  construct
  {
    this.set_name("Gwibber");
    this.set_icon_name("gwibber");
    //this.set_icon_from_file("/usr/share/gwibber/ui/gwibber.svg");
    this.set_title("Gwibber");
    this.destroy.connect(this.on_window_close);
    this.set_default_size(400,400);

    // colormap stuff
    this.set_colormap (Gdk.Screen.get_default ().get_rgba_colormap ());
    this.set_app_paintable (true);
    this.realize ();
    this.window.set_back_pixmap (null, false);

    // end colormap stuff

    
    /**** Create menus ****/
    var menubar = new Gtk.MenuBar();

    // Main Gwibber menu
    var gwibber_menu = new Gtk.Menu();
    var refresh_menu_item = new Gtk.MenuItem.with_mnemonic("_Refresh");
    refresh_menu_item.activate.connect(this.on_refresh);
    gwibber_menu.append(refresh_menu_item);

    var search_menu_item = new Gtk.MenuItem.with_mnemonic("_Search");
    search_menu_item.activate.connect(this.on_search);
    gwibber_menu.append(search_menu_item);

    var new_stream_menu_item = new Gtk.MenuItem.with_mnemonic("_New Stream");
    new_stream_menu_item.activate.connect(this.on_new_stream);
    gwibber_menu.append(new_stream_menu_item);

    var close_stream_menu_item = new Gtk.MenuItem.with_mnemonic("_Close Stream");
    close_stream_menu_item.activate.connect(this.on_close_stream);
    gwibber_menu.append(close_stream_menu_item);

    var quit_menu_item = new Gtk.MenuItem.with_mnemonic("_Quit");
    quit_menu_item.activate.connect(this.on_quit);
    gwibber_menu.append(quit_menu_item);

    var gwibber_launcher = new Gtk.MenuItem.with_mnemonic("_Gwibber");
    gwibber_launcher.set_submenu(gwibber_menu);
    menubar.append(gwibber_launcher);
    // End Main Gwibber menu

    // Edit menu
    var edit_menu = new Gtk.Menu();
    var accounts_menu_item = new Gtk.MenuItem.with_mnemonic("_Accounts");
    accounts_menu_item.activate.connect(this.on_accounts);
    edit_menu.append(accounts_menu_item);

    var preferences_menu_item = new Gtk.MenuItem.with_mnemonic("_Preferences");
    preferences_menu_item.activate.connect(this.on_preferences);
    edit_menu.append(preferences_menu_item);

    var edit_launcher = new Gtk.MenuItem.with_mnemonic("_Edit");
    edit_launcher.set_submenu(edit_menu);
    menubar.append(edit_launcher);
    // End Edit menu


    // Help menu
    var help_menu = new Gtk.Menu();
    var help_online_menu_item = new Gtk.MenuItem.with_mnemonic("_Get Help Online...");
    help_online_menu_item.activate.connect(this.load_questions_url);
    help_menu.append(help_online_menu_item);

    var translate_menu_item = new Gtk.MenuItem.with_mnemonic("_Translate This Application...");
    translate_menu_item.activate.connect(this.load_translate_url);
    help_menu.append(translate_menu_item);

    var bug_menu_item = new Gtk.MenuItem.with_mnemonic("_Report A Problem...");
    bug_menu_item.activate.connect(this.load_bug_url);
    help_menu.append(bug_menu_item);

    var about_menu_item = new Gtk.MenuItem.with_mnemonic("_About");
    about_menu_item.activate.connect(this.on_about);
    help_menu.append(about_menu_item);

    var help_launcher = new Gtk.MenuItem.with_mnemonic("_Help");
    help_launcher.set_submenu(help_menu);
    menubar.append(help_launcher);
    // End Help menu
    /**** End menu setup ****/
    

    var view = new GwibberGtk.StreamView ();
    /* This can be a Dee.SharedModel too */
    var model = create_model ();
    view.model = model;
    //view.set_size_request (350, 650);


    var main_box = new Gtk.VBox(false, 0);
    main_box.pack_start(menubar, false, true, 0);

    //var stream_vbox = new Gtk.VBox (false, 0);
    //var nav_vbox = new Gtk.VBox (false, 0);
    //stream_vbox.pack_start (view, true, true, 0);

    //var top_box = new Gtk.VBox(false, 0);
    //var bottom_box = new Gtk.VBox(false, 0);
    var entry = new GwibberGtk.Entry ();
    //bottom_box.add(entry);
    //bottom_box.set_size_request (350, 200);
    //var splitter = new Gtk.HPaned ();
    //splitter.pack1(nav_vbox, false, true);
    //splitter.pack2(stream_vbox, true, true);
    //splitter.pack2(view, true, false);
    //top_box.add(splitter);
    //var input_splitter = new Gtk.VPaned ();
    //input_splitter.pack1 (top_box, true, true);
    //input_splitter.pack2 (bottom_box, false, true);
    //main_box.pack_end(input_splitter, true, false, 2);

    main_box.pack_start (view, true, true);
    main_box.pack_start (entry, false, true);
    
    this.add(main_box);

    this.show_all ();
  }

  static void on_refresh() {
    debug ("on_refresh");
  }

  static void on_preferences() {
    debug ("on_preferences");
  }

  static void on_accounts() {
    debug ("on_accounts");
  }

  
  public override bool expose_event (Gdk.EventExpose event) {
    var cr = Gdk.cairo_create (this.window);

    cr.rectangle (event.area.x, event.area.y,
                  event.area.width, event.area.height);
    cr.clip ();
    cr.set_operator (Cairo.Operator.CLEAR);
    cr.paint ();

    cr.set_source_rgba (60/255.0f, 59/255.0f, 55/255.0f, 0.85f);
    cr.set_operator (Cairo.Operator.OVER);
    cr.paint ();

    propagate_expose (get_child (), event); 

    return true;
  }


  static void on_close_stream() {
    debug ("on_close_stream");
  }

  static void on_search() {
    debug ("on_search");
  }

  static void on_about() {
    debug ("on_about");
    var about_dialog = new Gtk.AboutDialog ();
    about_dialog.set_program_name("Gwibber");
    about_dialog.set_logo_icon_name("gwibber");
    about_dialog.set_version("2.91.1");
    about_dialog.set_website("http://www.gwibber.com");
    about_dialog.run ();
    about_dialog.destroy ();
  }

  static void on_quit() {
    debug ("on_quit");
    Gtk.main_quit();
  }

  static void on_new_stream() {
    debug ("on_new_stream");
  }

  static void on_window_close() {
    debug ("on_window_close");
    Gtk.main_quit();
  }

  static void load_questions_url() {
    debug ("load_questions_url");
  }

  static void load_translate_url() {
    debug ("load_translate_url");
  }

  static void load_bug_url() {
    debug ("load_bug_url");
  }
}

public static void main (string[] args)
{
  Gtk.init (ref args);

  var c = new Client ();

  c.show ();
  //w.resize (400, 800);
  c.position = Gtk.WindowPosition.CENTER;
  Gtk.main ();
}
